% (2015)
% 
% Author :
% Jeremy Schmitt (jeremy.schmitt@ens-lyon.fr) 
% --> Contact author Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% 
% Contributors :
% Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% Patrick Flandrin (patrick.flandrin@ens-lyon.fr)
% Pierre Borgnat (pierre.borgnat@ens-lyon.fr)
% Laurent Condat (Laurent.Condat@gipsa-lab.grenoble-inp.fr)
% 
% This software contains some image processing algorithms whose purpose is to be
% used primarily for research.
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software.  You can  use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability.
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.

function results = main_emd(image,K,varargin)

% The function main_emd is the main function for 2D mode decomposition. 
% It provides a 2D mode decomposition of the
% image of size (n,m). The decomposition is performed
% according to the work "2D Prony-Huang Transform: A New Tool for 2-D
% Spectral Analysis, J. Schmitt, N. Pustelnik, P. Borgnat, P. Fladrin, L.
% Condat, IEEE Transactions on Image Processing, 2014".
%
% The algorithm decomposes the data x into a trend component a_K (results.trend)
% and K IMF component d_k (results.imf).
% It consists to solve
% For k=1 : K-1
%     (a_k,d_k) = \arg\min_{a,d} ||x-a-d||_2^2 + lambda_geometry.||A d||_{2,1} + lambda.texture.||D d ||_1
%     x = a_k
% End
% The parameters involved in the criterion can be tuned in order to provide
% good estimates.
%
%
%%%%%%%%%%%%EXAMPLES%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    wave1 = generate_wave(10,0.5,pi/2,256);
%    wave2 = generate_wave(10,5,pi/4,256);
%    wave3 = generate_wave(10,40,pi/3,256);
%    image = wave1 + wave2 + wave3;
%    K=2;
%    lambda_geometry = [1 21];
%    lambda_texture = [1 1];
%    results = main_emd(image,K,lambda_geometry,lambda_texture);
%    
%%%%%%%%%%%%INPUTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% image : image to decompose
%
% K : number of IMFs to extract
%
% Optional : lambda_geometry : regularization parameters for trend 
%                              (vector of length nc) (default : 1)
%            lambda_texture : regularization parameters for IMF
%                             (vector of lenght nc) (default : 1)

%%%%%%%%%%%%OUTPUTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% results                : results of algorithm
% results.trend          : trend of order K
% results.imf            : 3D matrix (n x m x K) containinh containing the K IMFs
% results.crit           : 2D matrix (niter x K) containing the variation of the
%                          convergence criterion 
%                          ||x-a-d||_2^2 + lambda_geometry.||A d||_{2,1} + lambda.texture.||D d ||_1
%
%%%%%%%%%%%%SUBFUNCTIONS%%%%%%%%%%%%%%%%%%%%%%%%
%
% define_parameters_emd  : define regularization parameters of method
% 
% run_prox_emd_2D : performs mode decomposition

addpath( genpath('include'), '-end');


data.x = image;
data.x0 = data.x;
[nx,ny] = size(image);
data.n = nx;
data.m = ny;
clear nx ny;

lambda_geometry = zeros(1,K);
lambda_texture = zeros(1,K);

switch nargin
    case 2
        lambda_geometry(1:K) = 1;
        lambda_texture(1:K) = 1;
    case 3
        lambda_geometry(1:K) = varargin{1};
        lambda_texture(1:K) = 1;
    case 4
        lambda_geometry(1:K) = varargin{1};
        lambda_texture(1:K) = varargin{2};
end


%define parameters

lambda = define_parameters_emd_2D(lambda_geometry,lambda_texture,K);

%run algorithm

results = run_prox_emd_2D(data,lambda);

%plot results

%IMFs
figure(1);
subplot(1,lambda.K + 1,1);imagesc(results.trend);axis image off; colormap(gray);
title 'Trend'
for indi=1:lambda.K
    subplot(1,lambda.K+1,indi+1);imagesc(results.imf(:,:,indi));axis image off; colormap(gray);
    tit=['IMF ',num2str(indi)];
    title(tit)
end

%EMD convergence criterion
figure(2);
for indi=1:lambda.K
    subplot(1,lambda.K,indi);semilogy(results.crit(:,indi)); hold on;
    tit=['crit ',num2str(indi)];
    title(tit)
end


end